package com.chinacreator.ecd.changshipa.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.ecd.changshipa.bean.ChangshiPublicAccountChannel;
import com.chinacreator.ecd.changshipa.bean.ChangshiPublicAccountTemplate;
import com.chinacreator.ecd.changshipa.exception.BusinessException;
import com.chinacreator.ecd.changshipa.mapper.ChangshiPublicAccountMapper;
import com.chinacreator.ecd.changshipa.restrain.ReqEnum;
import com.chinacreator.ecd.changshipa.task.ChangshiPublicAccountTask;
import com.chinacreator.ecd.changshipa.util.DESUtil;
import com.chinacreator.ecd.changshipa.util.MD5;
import com.github.kevinsawicki.http.HttpRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 畅视公众号模板消息
 * 华盛接口
 */
@Service
public class ChangshiPublicAccountService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ChangshiPublicAccountService.class);

    /**
     * 测试地址
     */
    @Value("${changshi.publicaccount.template}")
    private String url;

    @Value("${changshi.publicaccount.password}")
    private String password;

    @Autowired
    private ChangshiPublicAccountMapper mapper;

    private List<ChangshiPublicAccountTemplate> templateList;

    private List<ChangshiPublicAccountChannel> channelList;

    @PostConstruct
    public void init() {
        ChangshiPublicAccountTask thread = new ChangshiPublicAccountTask(this);
        new Thread(thread).start();
    }

    /**
     * 通过畅视公众号订购的号码，调用华盛的接口，给用户发送通知消息
     * @return
     * @throws Exception
     */
    public Map<String, Object> send(Map<String, String> param) throws Exception {
        Map<String, Object> result = new HashMap<>();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        checkInput(param);

        String usermob = param.get("usermob");
        String elementid = param.get("elementid");
        String busiid = param.get("busiid");

        ChangshiPublicAccountTemplate template = selectTemplateByElementId(elementid, busiid);
        // 该产品未配置畅视公众号模板，不发送
        if (template == null) {
            LOGGER.info("该产品未配置模板，不发送模板通知：{}", param);
            return result;
        }

        if (template.getParams() == null) {
            LOGGER.info("该产品未配置模板的参数，不发送模板通知：{}", param);
            return result;
        }

        // 组装模板参数，即组装 vars 的取值
        Map<String, Object> varsMap = new HashMap<>();
        for (String name : template.getParams().split(",")) {
            String templateParamName = name.split("\\|")[0];
            String directedParamName = name.split("\\|")[1];

            if ("productname".equals(directedParamName)) {
                varsMap.put(templateParamName, template.getProductname());
            } else if ("price".equals(directedParamName)) {
                varsMap.put(templateParamName, template.getPrice());
            } else {
                varsMap.put(templateParamName, param.get(directedParamName));
            }
        }

        Map<String, Object> map = new HashMap<>();
        map.put("templateId", template.getTemplateid());
        map.put("mobile", DESUtil.encode(usermob, password));
        map.put("vars", JSON.toJSONString(varsMap));
        map.put("timestamp", System.currentTimeMillis() + "");
        map.put("signature", signature(map, password));

        HttpRequest request = HttpRequest.post(url)
                .contentType("application/json", "UTF-8").send(JSON.toJSONString(map));
        String resultString = new String(request.bytes(), Charset.forName("UTF-8"));

        LOGGER.info("畅视公众号模板消息发送：入参={}，返回={}", JSON.toJSONString(map), resultString);

        // {"code":0,"data":null,"message":"成功","success":true}
        JSONObject json = JSON.parseObject(resultString);
        if ("0".equals(json.getString("code"))) {
            return result;
        }

        result.put("resultCode", "8003");
        result.put("errorInfo", json.getString("message"));

        return result;
    }

    /**
     * 签名生成方式：signature= MD5(mobile&templateId&timestamp&passward)
     * &符号不用
     * @param map
     * @param password
     * @return
     */
    private static String signature(Map<String, Object> map, String password) {
        StringBuilder origin = new StringBuilder();
        origin.append(map.get("mobile").toString())
                .append(map.get("templateId").toString())
                .append(map.get("timestamp").toString())
                .append(password);

        return MD5.MD5Encode(origin.toString());
    }

    private static void checkInput(Map<String, String> param) throws BusinessException {
        Map<String, Object> map = new HashMap<>();

        String usermob = param.get("usermob");
//        String productid = param.get("productid");
        String busiid = param.get("busiid");
//        String orderchannel = param.get("orderchannel");
        String elementid = param.get("elementid");

        if (usermob == null || "".equals(usermob)) {
            throw new BusinessException(ReqEnum.ERROR_MISSING_REQUIRE_PARAMETER, "缺少必填参数【手机号码】！");
        }

//        if (productid == null || "".equals(productid)) {
//            throw new BusinessException(ReqEnum.ERROR_MISSING_REQUIRE_PARAMETER, "缺少必填参数【产品编码】！");
//        }

        if (elementid == null || "".equals(elementid)) {
            throw new BusinessException(ReqEnum.ERROR_MISSING_REQUIRE_PARAMETER, "缺少必填参数【elementid】！");
        }

        if (busiid == null || "".equals(busiid)) {
            throw new BusinessException(ReqEnum.ERROR_MISSING_REQUIRE_PARAMETER, "缺少必填参数【busiid】！");
        }

//        if (orderchannel == null || "".equals(orderchannel)) {
//            throw new BusinessException(ReqEnum.ERROR_MISSING_REQUIRE_PARAMETER, "缺少必填参数【渠道编码】！");
//        }
    }

    private ChangshiPublicAccountChannel selectChannel(String productid, String orderchannel) {
        ChangshiPublicAccountChannel channel = null;
        for (ChangshiPublicAccountChannel temp : channelList) {
            if (temp.getProductid().equals(productid) &&
                    temp.getOrderchannel().equals(orderchannel)) {
                channel = temp;
                break;
            }
        }
        return channel;
    }

    private ChangshiPublicAccountTemplate selectTemplate(String productid, String busiid) {
        ChangshiPublicAccountTemplate template = null;
        for (ChangshiPublicAccountTemplate temp : templateList) {
            if (temp.getProductid().equals(productid) &&
                    temp.getBusiid().equals(busiid)) {
                template = temp;
                break;
            }
        }
        return template;
    }

    private ChangshiPublicAccountTemplate selectTemplateByElementId(String elementid, String busiid) {
        ChangshiPublicAccountTemplate template = null;
        for (ChangshiPublicAccountTemplate temp : templateList) {
            if (temp.getElementid().equals(elementid) &&
                    temp.getBusiid().equals(busiid)) {
                template = temp;
                break;
            }
        }
        return template;
    }

    public void loadChangshiPublicAccount() {

        this.templateList = mapper.queryTemplates();

    }
}
